@@ -40,7 +40,6 @@
 
 #include "inlinepixbufs.h"
 #include "eel/eel-ellipsizing-label.h"
-#include "egg-screen-help.h"
 #include "wncklet.h"
 
 typedef struct {
@@ -71,7 +70,7 @@ window_menu_help (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	egg_help_display_desktop_on_screen (
+	gnome_help_display_desktop_on_screen (
 		NULL, "user-guide", "wgosmetacity.xml", "gosmetacity-27",
 		gtk_widget_get_screen (window_menu->applet),
 		&error);
