@@ -38,12 +38,10 @@
 #include "panel-toplevel.h"
 #include "panel-a11y.h"
 #include "panel-globals.h"
+#include "panel-multiscreen.h"
 
 #include "quick-desktop-reader.h"
 
-#include "egg-screen-exec.h"
-#include "egg-screen-url.h"
-
 #undef LAUNCHER_DEBUG
 
 static void properties_apply (Launcher *launcher);
@@ -101,6 +99,7 @@ panel_launcher_save_ditem (GnomeDesktopItem *ditem,
 		panel_error_dialog (screen,
 				    "cannot_save_launcher",
 				    _("Cannot save launcher to disk"),
+				    "%s",
 				    error->message);
 
 		g_error_free (error);
@@ -132,13 +131,14 @@ launch_url (Launcher *launcher)
 		return;
 	}
 
-	egg_url_show_on_screen (url, screen, &error);
+	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
 		panel_error_dialog (screen,
 				    "cannot_show_url_dialog",
 				    _("Cannot show %s"),
-				    error->message,
-				    url);
+				    "%s",
+				    url,
+				    error->message);
 
 		g_clear_error (&error);
 	}
@@ -170,6 +170,7 @@ launch_cb (GtkWidget *widget,
 			panel_error_dialog (launcher_get_screen (launcher),
 					    "cannot_launch_icon",
 					    _("Cannot launch icon"),
+					    "%s",
 					    error->message);
 
 			g_clear_error (&error);
@@ -196,13 +197,11 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       Launcher         *launcher)
 {
-	GError *error = NULL;
-	char **envp = NULL;
+	GdkScreen  *screen = launcher_get_screen (launcher);
+	GError     *error = NULL;
+	char      **envp = NULL;
 
-	GdkScreen *screen = launcher_get_screen (launcher);
-
-	if (gdk_screen_get_default () != screen)
-		envp = egg_screen_exec_environment (screen);
+	envp = panel_make_environment_for_screen (screen, NULL);
 
 	gnome_desktop_item_drop_uri_list_with_env (launcher->ditem,
 						   (const char *)selection_data->data,
@@ -215,6 +214,7 @@ drag_data_received_cb (GtkWidget        *widget,
 		panel_error_dialog (launcher_get_screen (launcher),
 				    "cannot_launch_icon",
 				    _("Cannot launch icon"),
+				    "%s",
 				    error->message);
 		g_clear_error (&error);
 	}
@@ -479,7 +479,6 @@ setup_button (Launcher *launcher)
 {
 	const char *comment;
 	const char *name;
-	const char *docpath;
 	char *str;
 	const char *icon;
 	char *unescaped_str;
@@ -510,25 +509,7 @@ setup_button (Launcher *launcher)
 	/* Setup icon */
 	icon = gnome_desktop_item_get_string (launcher->ditem,
 					      GNOME_DESKTOP_ITEM_ICON);
-	button_widget_set_pixmap (BUTTON_WIDGET (launcher->button), icon);
-
-	/* Setup help */
-	docpath = gnome_desktop_item_get_string (launcher->ditem,
-						 "X-GNOME-DocPath");
-
-	panel_applet_remove_callback (launcher->info, "help_on_app");
-
-	if (docpath && *docpath != '\0') {
-		char *title;
-
-		title = g_strdup_printf (_("Help on %s _Application"), name);
-
-		panel_applet_add_callback (launcher->info,
-					   "help_on_app",
-					   GTK_STOCK_HELP,
-					   title);
-		g_free (title);
-	}
+	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), sure_string (icon));
 }
 
 
@@ -620,10 +601,14 @@ window_response (GtkWidget *w, int response, gpointer data)
 		 * signal again
 		 */
 		g_signal_connect (G_OBJECT (launcher->dedit), "changed",
-				    G_CALLBACK (launcher_changed),
-				    launcher);
+				  G_CALLBACK (launcher_changed),
+				  launcher);
 
-		properties_apply (launcher);
+		/* resave launcher */
+		launcher_save (launcher);
+
+		/* Setup the button look */
+		setup_button (launcher);
 
 	} else {
 		gtk_widget_destroy (w);
@@ -644,6 +629,7 @@ create_properties_dialog (Launcher  *launcher,
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Launcher Properties"));
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	help = gtk_dialog_add_button (
@@ -674,6 +660,10 @@ create_properties_dialog (Launcher  *launcher,
 	g_signal_connect (launcher->dedit, "changed",
 			  G_CALLBACK (launcher_changed),
 			  launcher);
+	
+	panel_widget_register_open_dialog (PANEL_WIDGET 
+					   (launcher->info->widget->parent),
+					   dialog);
 
         g_signal_connect_swapped (launcher->dedit, "changed",
 				  G_CALLBACK (set_revert_sensitive),
@@ -702,6 +692,9 @@ void
 launcher_properties (Launcher  *launcher,
 		     GdkScreen *screen)
 {
+	if (panel_profile_get_inhibit_command_line ())
+		return;
+
 	if (launcher->prop_dialog != NULL) {
 		gtk_window_set_screen (
 			GTK_WINDOW (launcher->prop_dialog), screen);
@@ -735,7 +728,11 @@ load_launcher_applet (const char       *location,
 	if (!launcher->info)
 		return NULL;
 
-	if (!commie_mode)
+	if ( ! panel_profile_get_locked_down () &&
+	     ! panel_toplevel_get_locked_down (panel->toplevel) &&
+	     /* if the command line is inhibited, don't allow editting launchers,
+		since that would allow running arbitrary commands */
+	     ! panel_profile_get_inhibit_command_line ())
 		panel_applet_add_callback (launcher->info,
 					   "properties",
 					   GTK_STOCK_PROPERTIES,
@@ -839,9 +836,20 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	g_free (launcher_location);
 }
 
+static void
+ensure_item_localefiled (GnomeDesktopItem *ditem, const char *field)
+{
+	if (gnome_desktop_item_get_string (ditem, field) == NULL) {
+		const char *localized = gnome_desktop_item_get_localestring (ditem, field);
+		if (localized != NULL)
+			gnome_desktop_item_set_string (ditem, field, localized);
+	}
+}
+
 static void
 really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 {
+	GtkWidget *err_dialog;
 	GnomeDItemEdit *dedit = GNOME_DITEM_EDIT(data);
 	int pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "pos"));
 	PanelWidget *panel = g_object_get_data (G_OBJECT (dialog), "panel");
@@ -852,6 +860,45 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 
 		ditem = gnome_ditem_edit_get_ditem (dedit);
 
+		/* check for valid name */
+		if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
+			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
+						         "cannot_create_launcher",
+						         _("Cannot create launcher"),
+						         _("You have to specify a name."));
+			g_signal_connect_swapped (G_OBJECT (err_dialog),
+						  "destroy",
+						   G_CALLBACK (panel_pop_window_busy),
+						   G_OBJECT (dialog));
+			return;
+		}
+		
+
+		/* check for valid URL or command */
+		if ((gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_APPLICATION && 
+		     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC))) ||
+		    (gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_LINK &&
+		     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
+			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
+							 "cannot_create_launcher",
+							 _("Cannot create launcher"),
+							 _("You have to specify a valid URL or command."));
+			g_signal_connect_swapped (G_OBJECT (err_dialog),
+						  "destroy",
+						  G_CALLBACK (panel_pop_window_busy),
+						  G_OBJECT (dialog));
+			return;
+		}
+
+		/* Make sure we set the "C" locale strings to the terms we set
+		   here.  This is so that if the user logs into another locale
+		   they get their own description there rather then empty.  It
+		   is not the C locale however, but the user created this entry
+		   herself so it's OK */
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_NAME);
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME);
+		ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_COMMENT);
+
 		panel_launcher_save_ditem (ditem, gtk_window_get_screen (GTK_WINDOW (dialog)));
 		location = gnome_desktop_item_get_location (ditem);
 
@@ -864,10 +911,6 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 		return;
 	}
 
-	panel->create_launcher_dialog_list = 
-					g_slist_remove (panel->create_launcher_dialog_list, 
-							dialog);
-
 	gtk_widget_destroy (dialog);
 }
 
@@ -883,7 +926,7 @@ ask_about_launcher (const char  *file,
 
 	dialog = gtk_dialog_new_with_buttons (
 				_("Create Launcher"),
-				GTK_WINDOW (panel->toplevel),
+				NULL,
 				0 /* flags */,
 				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -892,6 +935,7 @@ ask_about_launcher (const char  *file,
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
@@ -900,12 +944,9 @@ ask_about_launcher (const char  *file,
 	dee = GNOME_DITEM_EDIT (gnome_ditem_edit_new ());
 	gtk_container_set_border_width (GTK_CONTAINER (dee), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-			    GTK_WIDGET (dee),
-			    TRUE, TRUE, GNOME_PAD_SMALL);
+			    GTK_WIDGET (dee), TRUE, TRUE, 0);
 
-	panel->create_launcher_dialog_list = 
-					g_slist_append (panel->create_launcher_dialog_list,
-							dialog);
+	panel_widget_register_open_dialog (panel, dialog);
 
 	ditem = gnome_desktop_item_new ();
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, file);
@@ -926,36 +967,13 @@ ask_about_launcher (const char  *file,
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
+	gtk_window_set_screen (GTK_WINDOW (dialog), gtk_widget_get_screen (GTK_WIDGET (panel)));
+
 	gtk_widget_show_all (dialog);
 
 	gnome_ditem_edit_grab_focus (dee);
 }
 
-static void
-ditem_set_icon (GnomeDesktopItem *ditem, const char *icon)
-{
-	if (icon != NULL &&
-	    icon[0] != G_DIR_SEPARATOR) {
-		char *full = gnome_desktop_item_find_icon (panel_icon_theme,
-							   icon,
-							   48 /* desired size */,
-							   0 /* flags */);
-		if (full != NULL) {
-			gnome_desktop_item_set_string (ditem,
-						       GNOME_DESKTOP_ITEM_ICON,
-						       full);
-			g_free (full);
-		} else {
-			gnome_desktop_item_set_string (ditem,
-						       GNOME_DESKTOP_ITEM_ICON,
-						       icon);
-		}
-	} else {
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_ICON,
-					       icon);
-	}
-}
-
 void
 panel_launcher_create_from_info (PanelToplevel *toplevel,
 				 int            position,
@@ -977,7 +995,7 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_NAME, name);
 	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
 
-	ditem_set_icon (ditem, icon);
+	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_ICON, icon);
 
 	if (exec_info) {
 		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, exec_or_uri);
@@ -996,9 +1014,9 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 }
 
 void
-panel_launcher_create (PanelToplevel *toplevel,
-		       int            position,
-		       const char    *location)
+panel_launcher_create_with_id (const char    *toplevel_id,
+			       int            position,
+			       const char    *location)
 {
 	GConfClient *client;
 	const char  *profile;
@@ -1010,14 +1028,23 @@ panel_launcher_create (PanelToplevel *toplevel,
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_LAUNCHER, toplevel, position, FALSE);
+	id = panel_profile_prepare_object_with_id (PANEL_OBJECT_LAUNCHER, toplevel_id, position, FALSE);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
 	gconf_client_set_string (client, key, location, NULL);
 
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+}
 
+void
+panel_launcher_create (PanelToplevel *toplevel,
+		       int            position,
+		       const char    *location)
+{
+	panel_launcher_create_with_id (panel_profile_get_toplevel_id (toplevel),
+				       position,
+				       location);
 }
 
 void
@@ -1089,29 +1116,6 @@ find_launcher (const char *path)
 	return NULL;
 }
 
-void
-launcher_show_help (Launcher  *launcher,
-		    GdkScreen *screen)
-{
-	GError     *error = NULL;
-	const char *docpath;
-
-	if (!launcher->ditem)
-		return;
-
-	docpath = gnome_desktop_item_get_string (
-				launcher->ditem, "X-GNOME-DocPath");
-	panel_show_gnome_kde_help (screen, docpath, &error);
-	if (error) {
-		panel_error_dialog (screen,
-				    "cannot_show_gnome_kde_help",
-				    _("Cannot display help document"),
-				    error->message);
-
-		g_error_free (error);
-	}
-}
-
 void
 panel_launcher_set_dnd_enabled (Launcher *launcher,
 				gboolean  dnd_enabled)
