@@ -31,7 +31,6 @@
 #include <libgnome/gnome-i18n.h>
 
 #include "applet.h"
-#include "egg-screen-exec.h"
 #include "menu.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
@@ -43,6 +42,9 @@
 #include "panel-session.h"
 #include "panel-globals.h"
 #include "panel-run-dialog.h"
+#include "panel-a11y.h"
+
+#define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
 enum {
 	PROP_0,
@@ -84,10 +86,15 @@ panel_action_lock_setup_menu (PanelActionButton *button)
 {
 	panel_applet_add_callback (button->priv->info, "activate", NULL, _("_Activate Screensaver"));
 	panel_applet_add_callback (button->priv->info, "lock",     NULL, _("_Lock Screen"));
-	panel_applet_add_callback (button->priv->info, "exit",     NULL, _("_Kill Screensaver Daemon"));
+
+	/* in commie mode don't allow killing the screensaver */
+	if ( ! panel_profile_get_locked_down ())
+		panel_applet_add_callback (button->priv->info, "exit",     NULL, _("_Kill Screensaver Daemon"));
+
 	panel_applet_add_callback (button->priv->info, "restart",  NULL, _("Restart _Screensaver Daemon"));
 
-	if (!commie_mode)
+	/* in commie mode don't allow changing the screensaver properties */
+	if ( ! panel_profile_get_locked_down ())
 		panel_applet_add_callback (button->priv->info, "prefs", NULL, _("_Properties"));
 }
 
@@ -97,21 +104,20 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 {
 	char *command = NULL;
 
-	if (!strcmp (callback_name, "restart"))
-		command = g_strdup ("xscreensaver-command -exit ; xscreensaver &");
-
-	else if (!strcmp (callback_name, "prefs"))
+	if (!strcmp (callback_name, "prefs"))
 		command = g_strdup ("xscreensaver-demo");
 
 	else if (!strcmp (callback_name, "activate") ||
 		 !strcmp (callback_name, "lock") ||
-		 !strcmp (callback_name, "exit"))
+		 !strcmp (callback_name, "exit") ||
+		 !strcmp (callback_name, "restart"))
 		command = g_strdup_printf ("xscreensaver-command -%s", callback_name);
 
 	if (command)
-		egg_screen_execute_shell (
+		gdk_spawn_command_line_on_screen (
 			gtk_widget_get_screen (GTK_WIDGET (button)),
-					       g_get_home_dir (), command);
+			command,
+			NULL);
 
 	g_free (command);
 }
@@ -138,6 +144,9 @@ panel_action_logout (GtkWidget *widget)
 void
 panel_action_run_program (GtkWidget *widget)
 {
+	if (panel_profile_get_inhibit_command_line ())
+		return;
+
 	panel_run_dialog_present (gtk_widget_get_screen (widget));
 }
 
@@ -147,14 +156,22 @@ void
 panel_action_search (GtkWidget *widget)
 {
 	GdkScreen *screen;
+	GError    *error = NULL;
 	char      *argv[2] = {"gnome-search-tool", NULL};
 
 	screen = gtk_widget_get_screen (widget);
 
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 1, argv) < 0)
+	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+				  G_SPAWN_SEARCH_PATH,
+				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_gnome-search-tool",
-				    _("Cannot execute gnome-search-tool"));
+				    _("Cannot execute '%s'"),
+				    "%s",
+				    "gnome-search-tool",
+				    error->message);
+		g_error_free (error);
+	}
 }
 
 /* Take Screenshot
@@ -163,14 +180,22 @@ void
 panel_action_screenshot (GtkWidget *widget)
 {
 	GdkScreen *screen;
+	GError    *error = NULL;
 	char      *argv [2] = {"gnome-panel-screenshot", NULL};
 
 	screen = gtk_widget_get_screen (widget);
 
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 1, argv) < 0)
+	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+				  G_SPAWN_SEARCH_PATH,
+				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_gnome-panel-screenshot",
-				    _("Cannot execute gnome-panel-screenshot"));
+				    _("Cannot execute '%s'"),
+				    "%s",
+				    "gnome-panel-screenshot",
+				    error->message);
+		g_error_free (error);
+	}
 }
 
 /* Force Quit
@@ -180,6 +205,9 @@ panel_action_force_quit (GtkWidget *widget)
 {
 	GdkScreen *screen;
 
+	if (panel_profile_get_inhibit_force_quit ())
+		return;
+
 	screen = gtk_widget_get_screen (widget);
 
 	panel_force_quit (screen);
@@ -214,22 +242,22 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_RUN, PANEL_STOCK_RUN,
-		N_("Run Application"), NULL,
+		N_("Run Application"), "gospanel-555",
 		panel_action_run_program, NULL, NULL
 	},
 	{
 		PANEL_ACTION_SEARCH, PANEL_STOCK_SEARCHTOOL,
-		N_("Search for Files"), NULL,
+		N_("Search for Files"), "gospanel-554",
 		panel_action_search, NULL, NULL
 	},
 	{
 		PANEL_ACTION_SCREENSHOT, PANEL_STOCK_SCREENSHOT,
-		N_("Screenshot"), NULL,
+		N_("Screenshot"), "gospanel-553",
 		panel_action_screenshot, NULL, NULL
 	},
 	{
 		PANEL_ACTION_FORCE_QUIT, PANEL_STOCK_FORCE_QUIT,
-		N_("Force Quit"), NULL,
+		N_("Force Quit"), "gospanel-563",
 		panel_action_force_quit, NULL, NULL
 	}
 };
@@ -246,9 +274,6 @@ panel_action_button_finalize (GObject *object)
 				    button->priv->gconf_notify);
 	button->priv->gconf_notify = 0;
 
-	g_free (button->priv);
-	button->priv = NULL;
-
 	parent_class->finalize (object);
 }
 
@@ -355,8 +380,7 @@ panel_action_button_clicked (GtkButton *gtk_button)
 }
 
 static void
-panel_action_button_class_init (PanelActionButtonClass *klass,
-				gpointer               dummy)
+panel_action_button_class_init (PanelActionButtonClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
@@ -372,6 +396,8 @@ panel_action_button_class_init (PanelActionButtonClass *klass,
 
 	button_class->clicked       = panel_action_button_clicked;
 
+	g_type_class_add_private (klass, sizeof (PanelActionButtonPrivate));
+
 	g_object_class_install_property (
 			gobject_class,
 			PROP_ACTION_TYPE,
@@ -386,7 +412,7 @@ panel_action_button_class_init (PanelActionButtonClass *klass,
 			gobject_class,
 			PROP_DND_ENABLED,
 			g_param_spec_boolean ("dnd-enabled",
-					      _("Drag N' Drop enabled"),
+					      _("Drag and drop enabled"),
 					      _("Whether or not drag and drop is enabled on the widget"),
 					      TRUE,
 					      G_PARAM_READWRITE));
@@ -396,7 +422,7 @@ static void
 panel_action_button_instance_init (PanelActionButton      *button,
 				   PanelActionButtonClass *klass)
 {
-	button->priv = g_new0 (PanelActionButtonPrivate, 1);
+	button->priv = PANEL_ACTION_BUTTON_GET_PRIVATE (button);
 
 	button->priv->type = PANEL_ACTION_NONE;
 	button->priv->info = NULL;
@@ -446,6 +472,7 @@ panel_action_button_set_type (PanelActionButton     *button,
 
 	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
 			      _(actions [type].tooltip), NULL);
+	panel_a11y_set_atk_name_desc (GTK_WIDGET (button), _(actions [type].tooltip), NULL);
 }
 
 static void
@@ -665,7 +692,7 @@ panel_action_button_load_from_drag (PanelToplevel *toplevel,
 		return retval;
 	}
 
-	if (!gconf_string_to_enum (panel_action_type_map, elements [1], (int *) &type)) {
+	if (!gconf_string_to_enum (panel_action_type_map, elements [1], (gpointer) &type)) {
 		g_strfreev (elements);
 		return retval;
 	}
