@@ -39,59 +39,56 @@
 #include "menu.h"
 #include "quick-desktop-reader.h"
 
+#define PANEL_MENU_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BUTTON, PanelMenuButtonPrivate))
+
 enum {
 	PROP_0,
 	PROP_MENU_PATH,
 	PROP_CUSTOM_ICON,
+	PROP_TOOLTIP,
 	PROP_USE_MENU_PATH,
-	PROP_USE_CUSTOM_ICON
+	PROP_USE_CUSTOM_ICON,
+	PROP_DND_ENABLED
 };
 
 struct _PanelMenuButtonPrivate {
-	AppletInfo            *info;
 	PanelToplevel         *toplevel;
 	guint                  gconf_notify;
+	char                  *applet_id;
 
 	GtkMenu               *menu;
 
 	char                  *menu_path;
 	char                  *custom_icon;
+	char                  *tooltip;
 
 	guint                  use_menu_path : 1;
 	guint                  use_custom_icon : 1;
+	guint                  dnd_enabled : 1;
 };
 
+static void panel_menu_button_disconnect_from_gconf (PanelMenuButton *button);
+
 static GObjectClass *parent_class;
 
 static void
 panel_menu_button_instance_init (PanelMenuButton      *button,
 				 PanelMenuButtonClass *klass)
 {
-	static GtkTargetEntry dnd_targets [] = {
-		{ "application/x-panel-applet-internal", 0, 0 }
-	};
+	button->priv = PANEL_MENU_BUTTON_GET_PRIVATE (button);
 
-	button->priv = g_new0 (PanelMenuButtonPrivate, 1);
-
-	button->priv->info         = NULL;
+	button->priv->applet_id    = NULL;
 	button->priv->toplevel     = NULL;
 	button->priv->gconf_notify = 0;
 
 	button->priv->menu_path   = NULL;
 	button->priv->custom_icon = NULL;
+	button->priv->tooltip = NULL;
 
 	button->priv->use_menu_path   = FALSE;
 	button->priv->use_custom_icon = FALSE;
 
-	button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_GNOME_LOGO);
-
-	GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
-	gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
-			     dnd_targets, 1,
-			     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
-
-	/* FIXME: set a tooltip */
+	button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_MAIN_MENU);
 }
 
 static void
@@ -99,14 +96,16 @@ panel_menu_button_finalize (GObject *object)
 {
 	PanelMenuButton *button = PANEL_MENU_BUTTON (object);
 
-	button->priv->info = NULL;
+	panel_menu_button_disconnect_from_gconf (button);
 
-	gconf_client_notify_remove (panel_gconf_get_client (), button->priv->gconf_notify);
-	button->priv->gconf_notify = 0;
+	if (button->priv->menu) {
+		/* detaching the menu will kill our reference */
+		gtk_menu_detach (button->priv->menu);
+		button->priv->menu = NULL;
+	}
 
-	if (button->priv->menu)
-		g_object_unref (button->priv->menu);
-	button->priv->menu = NULL;
+	g_free (button->priv->applet_id);
+	button->priv->applet_id = NULL;
 
 	g_free (button->priv->menu_path);
 	button->priv->menu_path = NULL;
@@ -114,8 +113,8 @@ panel_menu_button_finalize (GObject *object)
 	g_free (button->priv->custom_icon);
 	button->priv->custom_icon = NULL;
 
-	g_free (button->priv);
-	button->priv = NULL;
+	g_free (button->priv->tooltip);
+	button->priv->tooltip = NULL;
 
 	parent_class->finalize (object);
 }
@@ -139,12 +138,18 @@ panel_menu_button_get_property (GObject    *object,
 	case PROP_CUSTOM_ICON:
 		g_value_set_string (value, button->priv->custom_icon);
 		break;
+	case PROP_TOOLTIP:
+		g_value_set_string (value, button->priv->tooltip);
+		break;
 	case PROP_USE_MENU_PATH:
 		g_value_set_boolean (value, button->priv->use_menu_path);
 		break;
 	case PROP_USE_CUSTOM_ICON:
 		g_value_set_boolean (value, button->priv->use_custom_icon);
 		break;
+	case PROP_DND_ENABLED:
+		g_value_set_boolean (value, button->priv->dnd_enabled);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -170,12 +175,18 @@ panel_menu_button_set_property (GObject      *object,
 	case PROP_CUSTOM_ICON:
                 panel_menu_button_set_custom_icon (button, g_value_get_string (value));
                 break;
+	case PROP_TOOLTIP:
+		panel_menu_button_set_tooltip (button, g_value_get_string (value));
+		break;
 	case PROP_USE_MENU_PATH:
 		panel_menu_button_set_use_menu_path (button, g_value_get_boolean (value));
 		break;
 	case PROP_USE_CUSTOM_ICON:
 		panel_menu_button_set_use_custom_icon (button, g_value_get_boolean (value));
 		break;
+	case PROP_DND_ENABLED:
+		panel_menu_button_set_dnd_enabled (button, g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -210,6 +221,9 @@ panel_menu_button_parent_set (GtkWidget *widget,
 		button->priv->toplevel = NULL;
 
 	panel_menu_button_associate_panel (button);
+
+	if (GTK_WIDGET_CLASS (parent_class)->parent_set)
+		GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
 }
 
 static void
@@ -267,23 +281,51 @@ panel_menu_button_create_menu (PanelMenuButton *button)
 	return GTK_MENU (menu);
 }
 
-static void
+static void 
+panel_menu_button_menu_detacher	(GtkWidget *widget,
+				 GtkMenu   *menu)
+{
+ 	PanelMenuButton *button;
+
+	g_return_if_fail (PANEL_IS_MENU_BUTTON (widget));
+
+	button = PANEL_MENU_BUTTON (widget);
+
+	g_return_if_fail (button->priv->menu == menu);
+
+	/* just in case someone still owns a reference to the
+	   menu (the menu may be up or some such other nonsense) */
+	g_signal_handlers_disconnect_by_func (button->priv->menu,
+					      G_CALLBACK (panel_menu_button_menu_deactivated),
+					      button);
+
+	/* This is a workaround pending fixing bug #113112 */
+	g_object_set_data (G_OBJECT (button), "gtk-attached-menu", NULL);
+	button->priv->menu = NULL;
+}
+
+void
 panel_menu_button_popup_menu (PanelMenuButton *button,
 			      guint            n_button,
 			      guint32          activate_time)
 {
 	GdkScreen *screen;
 
+	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
+
 	if (button->priv->menu && menu_need_reread (GTK_WIDGET (button->priv->menu))) {
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 		button->priv->menu = NULL;
 	}
 
 	if (!button->priv->menu) {
 		button->priv->menu = panel_menu_button_create_menu (button);
 
-		g_object_ref (button->priv->menu);
-		gtk_object_sink (GTK_OBJECT (button->priv->menu));
+		gtk_menu_attach_to_widget (button->priv->menu, 
+					   GTK_WIDGET (button),
+					   panel_menu_button_menu_detacher);
+		/* This is a workaround pending fixing bug #113112 */
+		g_object_set_data (G_OBJECT (button), "gtk-attached-menu", button->priv->menu);
 
 		panel_menu_button_associate_panel (button);
 
@@ -308,41 +350,6 @@ panel_menu_button_popup_menu (PanelMenuButton *button,
 			activate_time);
 }
 
-static void
-panel_menu_button_handle_event (PanelMenuButton *button)
-{
-	GdkEvent *event;
-	guint     n_button;
-	guint32   activate_time;
-
-	event = gtk_get_current_event ();
-
-	switch (event->type) {
-	case GDK_BUTTON_PRESS:
-	case GDK_2BUTTON_PRESS:
-	case GDK_3BUTTON_PRESS:
-	case GDK_BUTTON_RELEASE:
-		activate_time = event->button.time;
-		n_button = event->button.button;
-		break;
-	case GDK_KEY_PRESS:
-	case GDK_KEY_RELEASE:
-		activate_time = event->key.time;
-		n_button = 1;
-		break;
-	default:
-		activate_time = n_button = 0;
-		g_assert_not_reached ();
-		break;
-	}
-	
-	gtk_grab_remove (GTK_WIDGET (button));
-
-	panel_menu_button_popup_menu (button, n_button, activate_time);
-
-	gdk_event_free (event);
-}
-
 static void
 panel_menu_button_pressed (GtkButton *gtk_button)
 {
@@ -355,13 +362,14 @@ panel_menu_button_pressed (GtkButton *gtk_button)
 	if (GTK_BUTTON_CLASS (parent_class)->pressed)
 		GTK_BUTTON_CLASS (parent_class)->pressed (gtk_button);
 
-	panel_menu_button_handle_event (button);
+	panel_menu_button_popup_menu (button, 0, gtk_get_current_event_time());
 }
 
 static void
 panel_menu_button_clicked (GtkButton *gtk_button)
 {
 	PanelMenuButton *button;
+	GdkEvent        *event;
 
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (gtk_button));
 
@@ -370,12 +378,18 @@ panel_menu_button_clicked (GtkButton *gtk_button)
 	if (GTK_BUTTON_CLASS (parent_class)->clicked)
 		GTK_BUTTON_CLASS (parent_class)->clicked (gtk_button);
 
-	panel_menu_button_handle_event (button);
+	if ((event = gtk_get_current_event ())) {
+		panel_menu_button_popup_menu (button,
+					      event->button.button,
+					      event->button.time);
+		gdk_event_free (event);
+	} else {
+		panel_menu_button_popup_menu (button, 1, GDK_CURRENT_TIME);
+	}
 }
 
 static void
-panel_menu_button_class_init (PanelMenuButtonClass *klass,
-			   gpointer           dummy)
+panel_menu_button_class_init (PanelMenuButtonClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass   *) klass;
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
@@ -393,6 +407,8 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass,
 	button_class->clicked = panel_menu_button_clicked;
 	button_class->pressed = panel_menu_button_pressed;
 
+	g_type_class_add_private (klass, sizeof (PanelMenuButtonPrivate));
+
 	g_object_class_install_property (
 			gobject_class,
 			PROP_MENU_PATH,
@@ -411,6 +427,15 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass,
 					     NULL,
 					     G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+			gobject_class,
+			PROP_TOOLTIP,
+			g_param_spec_string ("tooltip",
+					     _("Tooltip"),
+					     _("Tooltip displayed for the menu"),
+					     NULL,
+					     G_PARAM_READWRITE));
+
 	g_object_class_install_property (
 			gobject_class,
 			PROP_USE_MENU_PATH,
@@ -425,9 +450,18 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass,
 			PROP_USE_CUSTOM_ICON,
                         g_param_spec_boolean ("use-custom-icon",
 					      _("Use Custom Icon"),
-					     _("Use the icon specified by the custom-icon property"),
-					     FALSE,
-					     G_PARAM_READWRITE));
+					      _("Use the icon specified by the custom-icon property"),
+					      FALSE,
+					      G_PARAM_READWRITE));
+
+	g_object_class_install_property (
+			gobject_class,
+			PROP_DND_ENABLED,
+                        g_param_spec_boolean ("dnd-enabled",
+					      _("Drag and drop enabled"),
+					      _("Whether or not drag and drop is enabled on the widget"),
+					      FALSE,
+					      G_PARAM_READWRITE));
 }
 
 GType
@@ -477,6 +511,10 @@ panel_menu_button_gconf_notify (GConfClient     *client,
 		if (value && value->type == GCONF_VALUE_STRING)
 			panel_menu_button_set_custom_icon (button,
 							   gconf_value_get_string (value));
+	} else if (!strcmp (key, "tooltip")) {
+		if (value && value->type == GCONF_VALUE_STRING)
+			panel_menu_button_set_tooltip (button,
+						       gconf_value_get_string (value));
 	} else if (!strcmp (key, "use_menu_path")) {
 		if (value && value->type == GCONF_VALUE_BOOL)
 			panel_menu_button_set_use_menu_path (button,
@@ -500,7 +538,7 @@ panel_menu_button_connect_to_gconf (PanelMenuButton *button)
 
 	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s",
 				   profile,
-				   button->priv->info->id);
+				   button->priv->applet_id);
 	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	button->priv->gconf_notify =
 		gconf_client_notify_add (client, key,
@@ -508,37 +546,69 @@ panel_menu_button_connect_to_gconf (PanelMenuButton *button)
 					 button, NULL, NULL);
 }
 
+static void
+panel_menu_button_disconnect_from_gconf (PanelMenuButton *button)
+{
+	GConfClient *client;
+	const char  *key;
+	const char  *profile;
+
+	if (!button->priv->gconf_notify)
+		return;
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+
+	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s",
+				   profile,
+				   button->priv->applet_id);
+
+	gconf_client_notify_remove (client, button->priv->gconf_notify);
+	button->priv->gconf_notify = 0;
+
+	gconf_client_remove_dir (client, key, NULL);
+}
+
 static void
 panel_menu_button_load (const char  *menu_path,
 			gboolean     use_menu_path,
 			const char  *custom_icon,
 			gboolean     use_custom_icon,
+			const char  *tooltip,
 			PanelWidget *panel,
+			gboolean     locked,
 			int          position,
 			gboolean     exactpos,
 			const char  *id)
 {
 	PanelMenuButton *button;
+	AppletInfo      *info;
 
 	g_return_if_fail (panel != NULL);
 
 	button = g_object_new (PANEL_TYPE_MENU_BUTTON,
 			       "menu-path", menu_path,
 			       "custom-icon", custom_icon,
+			       "tooltip", tooltip,
 			       "use-menu-path", use_menu_path,
 			       "use-custom-icon", use_custom_icon,
+			       "has-arrow", TRUE,
 			       NULL);
 
-	button->priv->info = panel_applet_register (
-					GTK_WIDGET (button), NULL, NULL, panel,
-					position, exactpos, PANEL_OBJECT_MENU, id);
-	if (!button->priv->info) {
+	info = panel_applet_register (GTK_WIDGET (button), NULL, NULL,
+				      panel, locked, position, exactpos,
+				      PANEL_OBJECT_MENU, id);
+	if (!info) {
 		gtk_widget_destroy (GTK_WIDGET (button));
 		return;
 	}
 
-	panel_applet_add_callback (
-		button->priv->info, "help", GTK_STOCK_HELP, _("_Help"));
+	button->priv->applet_id = g_strdup (info->id);
+
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"));
+
+	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
+	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
 
 	panel_menu_button_connect_to_gconf (button);
 }
@@ -563,7 +633,7 @@ panel_menu_button_get_icon_for_menu_path (PanelMenuButton *button)
 	if (!item)
 		return NULL;
                                                                                                              
-	retval = gnome_desktop_item_find_icon (panel_icon_theme, item->icon, 20, 0);
+	retval = g_strdup (item->icon);
                                                                                                              
 	quick_desktop_item_destroy (item);
 
@@ -576,15 +646,14 @@ panel_menu_button_set_icon (PanelMenuButton *button)
 	char *icon_path = NULL;
 
 	if (button->priv->use_custom_icon)
-		icon_path = button->priv->custom_icon;
+		icon_path = g_strdup (button->priv->custom_icon);
 
 	if (!icon_path)
 		icon_path = panel_menu_button_get_icon_for_menu_path (button);
 
-	if (icon_path)
-		button_widget_set_pixmap (BUTTON_WIDGET (button), icon_path);
-	else
-		button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_GNOME_LOGO);
+	button_widget_set_icon_name (BUTTON_WIDGET (button), icon_path);
+
+	g_free (icon_path);
 }
                                                                                                              
 void
@@ -607,7 +676,7 @@ panel_menu_button_set_menu_path (PanelMenuButton *button,
 		button->priv->menu_path = g_strdup (menu_path);
 
 	if (button->priv->menu)
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 	button->priv->menu = NULL;
 
 	panel_menu_button_set_icon (button);
@@ -628,6 +697,22 @@ panel_menu_button_set_custom_icon (PanelMenuButton *button,
 	panel_menu_button_set_icon (button);
 }
 
+void
+panel_menu_button_set_tooltip (PanelMenuButton *button,
+			       const char      *tooltip)
+{
+	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
+
+	g_free (button->priv->tooltip);
+	button->priv->tooltip = NULL;
+
+	if (tooltip && tooltip [0]) {
+		button->priv->tooltip = g_strdup (tooltip);
+		gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
+				      tooltip, NULL);
+	}
+}
+
 void
 panel_menu_button_set_use_menu_path (PanelMenuButton *button,
 				     gboolean         use_menu_path)
@@ -642,12 +727,20 @@ panel_menu_button_set_use_menu_path (PanelMenuButton *button,
 	button->priv->use_menu_path = use_menu_path;
 
 	if (button->priv->menu)
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 	button->priv->menu = NULL;
 
 	panel_menu_button_set_icon (button);
 }
 
+gboolean
+panel_menu_button_get_use_menu_path (PanelMenuButton *button)
+{
+	g_return_val_if_fail (PANEL_IS_MENU_BUTTON (button), FALSE);
+
+	return button->priv->use_menu_path;
+}
+
 void
 panel_menu_button_set_use_custom_icon (PanelMenuButton *button,
 				       gboolean         use_custom_icon)
@@ -661,6 +754,7 @@ panel_menu_button_set_use_custom_icon (PanelMenuButton *button,
 
 void
 panel_menu_button_load_from_gconf (PanelWidget *panel,
+				   gboolean     locked,
 				   int          position,
 				   gboolean     exactpos,
 				   const char  *id)
@@ -670,21 +764,22 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 	const char  *key;
 	char        *menu_path;
 	char        *custom_icon;
+	char        *tooltip;
 	gboolean     use_menu_path;
 	gboolean     use_custom_icon;
 
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s", profile, id);
-	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "menu_path");
 	menu_path = gconf_client_get_string (client, key, NULL);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "custom_icon");
 	custom_icon = gconf_client_get_string (client, key, NULL);
 
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "tooltip");
+	tooltip = gconf_client_get_string (client, key, NULL);
+
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "use_menu_path");
 	use_menu_path = gconf_client_get_bool (client, key, NULL);
 
@@ -695,20 +790,24 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 				use_menu_path,
 				custom_icon,
 				use_custom_icon,
+				tooltip,
 				panel,
+				locked,
 				position,
 				exactpos,
 				id);
 
 	g_free (menu_path);
 	g_free (custom_icon);
+	g_free (tooltip);
 }
 
 void
 panel_menu_button_create (PanelToplevel *toplevel,
 			  int            position,
 			  const char    *menu_path,
-			  gboolean       use_menu_path)
+			  gboolean       use_menu_path,
+			  const char    *tooltip)
 {
 	GConfClient *client;
 	const char  *profile;
@@ -718,16 +817,19 @@ panel_menu_button_create (PanelToplevel *toplevel,
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_MENU, toplevel, position);
+	id = panel_profile_prepare_object (PANEL_OBJECT_MENU, toplevel, position, FALSE);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "use_menu_path");
 	gconf_client_set_bool (client, key, use_menu_path, NULL);
 
-	if (menu_path) {
-		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "use_menu_path");
+	if (use_menu_path && menu_path && menu_path [0]) {
+		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "menu_path");
 		gconf_client_set_string (client, key, menu_path, NULL);
 	}
 
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "tooltip");
+	gconf_client_set_string (client, key, tooltip, NULL);
+
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
 }
@@ -750,10 +852,28 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 }
 
 void
-panel_menu_button_change_orientation (PanelMenuButton  *button,
-				      PanelOrientation  orientation)
+panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
+				   gboolean         dnd_enabled)
 {
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
 
-	button_widget_set_params (BUTTON_WIDGET (button), TRUE, orientation);
+	dnd_enabled = dnd_enabled != FALSE;
+
+	if (button->priv->dnd_enabled == dnd_enabled)
+		return;
+
+	if (dnd_enabled) {
+		static GtkTargetEntry dnd_targets [] = {
+			{ "application/x-panel-applet-internal", 0, 0 }
+		};
+
+		GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
+		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
+				     dnd_targets, 1,
+				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+		gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
+						PANEL_STOCK_MAIN_MENU);
+		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
+	} else
+		gtk_drag_source_unset (GTK_WIDGET (button));
 }
