@@ -31,7 +31,6 @@
 #include "egg-recent-view.h"
 #include "egg-recent-view-gtk.h"
 #include "egg-recent-item.h"
-#include "egg-screen-exec.h"
 #include "menu.h"
 #include "menu-util.h"
 #include "panel-util.h"
@@ -115,7 +114,7 @@ show_uri (const char *uri, const char *mime_type, GdkScreen *screen,
 	cmd = g_string_free (str, FALSE);
 
 	if (cmd != NULL &&
-	    !egg_screen_execute_command_line_async (screen, cmd, error)) {
+	    !gdk_spawn_command_line_on_screen (screen, cmd, error)) {
 		ret = FALSE;
 	}
 
@@ -127,10 +126,13 @@ show_uri (const char *uri, const char *mime_type, GdkScreen *screen,
 
 static void
 recent_documents_activate_cb (EggRecentViewGtk *view, EggRecentItem *item,
-			      GdkScreen *screen)
+			      GtkWidget *widget)
 {
 	char   *uri, *uri_utf8, *mime_type;
 	GError *error = NULL;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
 	
 	uri = egg_recent_item_get_uri (item);
 	uri_utf8 = egg_recent_item_get_uri_utf8 (item);
@@ -141,6 +143,7 @@ recent_documents_activate_cb (EggRecentViewGtk *view, EggRecentItem *item,
 			panel_error_dialog (screen,
 					    "cannot_open_recent_doc",
 					    _("Cannot open recently used document"),
+					    "%s",
 					    error->message);
 			g_error_free (error);
 		} else
@@ -163,13 +166,12 @@ panel_recent_append_documents_menu (GtkWidget *top_menu)
 	GtkWidget        *menu_item;
 	EggRecentModel   *model;
 	EggRecentViewGtk *view;
-	GdkScreen        *screen;
 
 	menu_item = gtk_image_menu_item_new ();
 	setup_stock_menu_item (menu_item,
 			       panel_menu_icon_get_size (),
 			       GTK_STOCK_OPEN,
-			       _("Open Recent"),
+			       _("Recent Documents"),
 			       TRUE);
 	menu = panel_create_menu ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
@@ -181,14 +183,13 @@ panel_recent_append_documents_menu (GtkWidget *top_menu)
 	gtk_widget_show_all (menu_item);
 	
 	/* a model that shows the global recent doc list */
-	model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
+	model = egg_recent_model_new ("gnome-panel", EGG_RECENT_MODEL_SORT_MRU);
 
 	view = egg_recent_view_gtk_new (menu, NULL);
 
-	screen = gtk_widget_get_screen (menu);
 	g_signal_connect (view, "activate",
 			  G_CALLBACK (recent_documents_activate_cb),
-			  screen);
+			  menu);
 	egg_recent_view_gtk_show_numbers (view, FALSE);
 	egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
 	egg_recent_view_gtk_set_icon_size (view, panel_menu_icon_get_size ());
