@@ -43,8 +43,6 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <gconf/gconf-client.h>
 
-#include "egg-screen-exec.h"
-
 #include "nothing.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
@@ -69,15 +67,13 @@ typedef struct {
 	GtkWidget        *pixmap;
 	GtkWidget        *run_button;
 	GtkWidget        *file_button;
-	GtkWidget        *list_checkbox;
+	GtkWidget        *list_expander;
 	GtkWidget        *terminal_checkbox;
 	GtkWidget        *program_label;
 	GtkWidget        *program_list;
 	
 	GtkListStore     *program_list_store;
 
-	GtkFileSelection *file_sel;
-	
 	GList            *executables;
 	GCompletion      *completion;
 	
@@ -257,7 +253,6 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	gboolean    result;	
 	GError     *error = NULL;
 	char      **argv;
-	char      **envp;
 	int         argc;
 	
 	if (!command_is_executable (command))
@@ -270,31 +265,31 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	argv [2] = g_strdup (command);
 	
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));	
-	envp = egg_screen_exec_environment (screen);
 		
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)))
 		gnome_prepend_terminal_to_vector (&argc, &argv);
 		   
-	result = g_spawn_async (NULL, /* working directory */
-				argv,
-				envp,
-				G_SPAWN_SEARCH_PATH,
-				NULL, /* child setup func */
-				NULL, /* user data */
-				NULL, /* child pid */
-				&error);
+	result = gdk_spawn_on_screen (screen,
+				      NULL, /* working directory */
+				      argv,
+				      NULL, /* envp */
+				      G_SPAWN_SEARCH_PATH,
+				      NULL, /* child setup func */
+				      NULL, /* user data */
+				      NULL, /* child pid */
+				      &error);
 			
 	if (!result) {
 		panel_error_dialog (screen, "cannot_spawn_command",
 				    _("Cannot launch command '%s'"),
-				    error->message,
-				    escaped);
+				    "%s",
+				    escaped,
+				    error->message);
 
 		g_error_free (error);
 	}
 				
 	g_strfreev (argv);
-	g_strfreev (envp);
 	
 	return result;
 }
@@ -304,22 +299,18 @@ panel_run_dialog_show_url (PanelRunDialog *dialog,
 	                   const char     *url,
 	                   const char     *escaped)
 {
-	GError     *error = NULL;
-	GdkScreen  *screen;
-	char      **envp;
+	GError    *error = NULL;
+	GdkScreen *screen;
 	
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
-	envp = egg_screen_exec_environment (screen);
-	
-	gnome_url_show_with_env (url, envp, &error);
-
-	g_strfreev (envp);
 
+	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
 		panel_error_dialog (screen, "cannot_show_url",
 				    _("Cannot display location '%s'"),
-				    error->message,
-				    escaped);
+				    "%s",
+				    escaped,
+				    error->message);
 
 		g_error_free (error);
 		return FALSE;
@@ -332,6 +323,7 @@ static void
 panel_run_dialog_execute (PanelRunDialog *dialog)
 {
 	GdkScreen *screen;
+	GError    *error;
 	gboolean   result;
 	char      *command;
 	char      *escaped;
@@ -362,7 +354,20 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 		return;
 	}
 		
-	disk = g_locale_from_utf8 (command, -1, NULL, NULL, NULL);
+	error = NULL;
+	disk = g_locale_from_utf8 (command, -1, NULL, NULL, &error);
+
+	if (!disk || error) {
+		panel_error_dialog (screen, "cannot_convert_command_from_utf8",
+				    _("Cannot convert '%s' from UTF-8"),
+				    "%s",
+				    command,
+				    error->message);
+
+		g_error_free (error);
+		return;
+	}
+
 	url = gnome_vfs_make_uri_from_input_with_dirs (disk,
 						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
 	escaped = g_markup_escape_text (url, -1);
@@ -404,6 +409,9 @@ panel_run_dialog_response (PanelRunDialog *dialog,
 			   int             response,
 			   GtkWidget      *run_dialog)
 {
+
+	dialog->completion_started = FALSE;
+
 	switch (response) {
 	case GTK_RESPONSE_OK:
 		panel_run_dialog_execute (dialog);
@@ -837,7 +845,7 @@ program_list_selection_changed (GtkTreeSelection *selection,
 
 	if (uri) {
 		ditem = gnome_desktop_item_new_from_uri (uri,
-							 GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS,
+							 GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,
 							 NULL /* error */);
 		if (ditem) {
 			dialog->use_program_list = TRUE;
@@ -859,7 +867,7 @@ program_list_selection_changed (GtkTreeSelection *selection,
 			temp = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_ICON);
 			panel_run_dialog_set_icon (dialog, temp);
 			
-			temp = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_COMMENT);
+			temp = gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT);
 			gtk_label_set_text (GTK_LABEL (dialog->program_label), sure_string (temp));
 			
 			terminal = gnome_desktop_item_get_boolean (ditem, GNOME_DESKTOP_ITEM_TERMINAL);
@@ -937,35 +945,18 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 				 gboolean        show_list)
 {
 	if (!panel_profile_get_enable_program_list ()) {
-		if (dialog->program_list_box->parent)
-			gtk_container_remove (GTK_CONTAINER (dialog->program_list_box->parent),
-					      dialog->program_list_box);
-
-		if (dialog->list_checkbox->parent)
-			gtk_container_remove (GTK_CONTAINER (dialog->list_checkbox->parent),
-					      dialog->list_checkbox);
+		if (dialog->list_expander->parent)
+			gtk_container_remove (GTK_CONTAINER (dialog->list_expander->parent),
+					      dialog->list_expander);
 		
-		return;
 	}
-
+	
 	if (show_list) {
-		if (!dialog->program_list_box->parent) {
-			gtk_box_pack_start (GTK_BOX (dialog->main_box),
-					    dialog->program_list_box,
-					    TRUE, TRUE, 5);
-                
-			gtk_widget_show_all (dialog->main_box);
-		}
-
 		gtk_window_resize (GTK_WINDOW (dialog->run_dialog), 100, 300);
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), TRUE);
 		gtk_widget_grab_focus (dialog->program_list);
 		
         } else if (!show_list) {
-		if (dialog->program_list_box->parent)
-			gtk_container_remove (GTK_CONTAINER (dialog->program_list_box->parent),
-					      dialog->program_list_box);
-		
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
                 gtk_widget_grab_focus (dialog->gtk_entry);
         }
@@ -981,30 +972,31 @@ panel_run_dialog_content_notify (GConfClient    *client,
 }
 
 static void
-list_checkbox_toggled (GtkToggleButton *button,
+list_expander_toggled (GtkExpander    *expander,
+		       GParamSpec     *pspec,
 		       PanelRunDialog *dialog)
 {
-	panel_profile_set_show_program_list (gtk_toggle_button_get_active (button));
+	panel_profile_set_show_program_list (gtk_expander_get_expanded (expander));
 }
 
 static void
-panel_run_dialog_setup_list_checkbox (PanelRunDialog *dialog,
+panel_run_dialog_setup_list_expander (PanelRunDialog *dialog,
 				      GladeXML       *gui)
 {
 	GConfClient *client;
 	const char *key;
 	
-	dialog->list_checkbox = glade_xml_get_widget (gui, "list_checkbox");
+	dialog->list_expander = glade_xml_get_widget (gui, "list_expander");
 
 	if (panel_profile_get_enable_program_list ()) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->list_checkbox),
-					      panel_profile_get_show_program_list ());
+		gtk_expander_set_expanded (GTK_EXPANDER (dialog->list_expander),
+					   panel_profile_get_show_program_list ());
 
 		if ( ! panel_profile_is_writable_show_program_list ())
-			gtk_widget_set_sensitive (dialog->list_checkbox, FALSE);
+			gtk_widget_set_sensitive (dialog->list_expander, FALSE);
 		
-	        g_signal_connect (dialog->list_checkbox, "toggled",
-				  G_CALLBACK (list_checkbox_toggled),
+	        g_signal_connect (dialog->list_expander, "notify::expanded",
+				  G_CALLBACK (list_expander_toggled),
 				  dialog);
 	
 		client = panel_gconf_get_client ();
@@ -1021,53 +1013,46 @@ panel_run_dialog_setup_list_checkbox (PanelRunDialog *dialog,
 }
 
 static void
-file_button_browse_ok (GtkWidget      *button,
-		       PanelRunDialog *dialog)
+file_button_browse_response (GtkWidget      *chooser,
+			     gint            response,
+			     PanelRunDialog *dialog)
 {
-	const char *file;
+	char *file;
 	
-	file = gtk_file_selection_get_filename (dialog->file_sel);
-	panel_run_dialog_append_file (dialog, file);
-	
-	gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
-	dialog->file_sel = NULL;
+	if (response == GTK_RESPONSE_OK) {
+		file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+		panel_run_dialog_append_file (dialog, file);
+		g_free (file);
+	}
 
-	gtk_widget_grab_focus (dialog->gtk_entry);
-}
-
-static void
-file_button_browse_cancel (GtkWidget      *button,
-			   PanelRunDialog *dialog)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
-	dialog->file_sel = NULL;
-
-	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_destroy (chooser);
+ 
+ 	gtk_widget_grab_focus (dialog->gtk_entry);
 }
 
 static void
 file_button_clicked (GtkButton      *button,
 		     PanelRunDialog *dialog)
 {
-	char *home;
+	GtkWidget *chooser;
 
-	dialog->file_sel = GTK_FILE_SELECTION (gtk_file_selection_new (
-						       _("Choose a file to append to the command...")));
+	chooser = gtk_file_chooser_dialog_new (_("Choose a file to append to the command..."),
+					       GTK_WINDOW (dialog->run_dialog),
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
 	
-	home = g_strconcat (g_get_home_dir (), "/", NULL);
-	gtk_file_selection_set_filename (dialog->file_sel, home);
-	g_free (home);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+					     g_get_home_dir ());
 	
-	gtk_window_set_transient_for (GTK_WINDOW (dialog->file_sel),
-				      GTK_WINDOW (dialog->run_dialog));
-				      
-	g_signal_connect (dialog->file_sel->ok_button, "clicked",
-			  G_CALLBACK (file_button_browse_ok), dialog);
-			  
-	g_signal_connect (dialog->file_sel->cancel_button, "clicked",
-		 	  G_CALLBACK (file_button_browse_cancel), dialog);
+	gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_OK);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (chooser), TRUE);
 
-	gtk_window_present (GTK_WINDOW (dialog->file_sel));
+	g_signal_connect (chooser, "response",
+			  G_CALLBACK (file_button_browse_response), dialog);
+
+	gtk_window_present (GTK_WINDOW (chooser));
 }
 
 static void
@@ -1088,6 +1073,7 @@ panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
 	struct dirent *dent;
 	DIR           *dir;
 	char          *file;
+	struct stat    file_stat;
 
 	dir = opendir (dirname);
 
@@ -1097,7 +1083,8 @@ panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
 	while ((dent = readdir (dir))) {
 		file = g_build_filename (dirname, dent->d_name, NULL);
 
-		if (!access (file, X_OK))
+		if (!access (file, X_OK) &&
+		    !stat (file, &file_stat) && !S_ISDIR (file_stat.st_mode))
 			dialog->executables = g_list_prepend (dialog->executables,
 							      g_strdup (dent->d_name));
 		
@@ -1152,7 +1139,6 @@ entry_event (GtkEditable    *entry,
 	if (event->type == GDK_KEY_PRESS)
 		dialog->use_program_list = FALSE;
 
-	dialog->completion_started = FALSE;
 
 	/* tab completion */
 	if (event->type == GDK_KEY_PRESS &&
@@ -1181,9 +1167,13 @@ entry_event (GtkEditable    *entry,
 			temp = gtk_editable_get_chars (entry, 0, pos);
 			prefix = g_strconcat (temp, event->string, NULL);
 			g_free (temp);
-		} else
+		} else if (pos == tmp &&
+			   tmp == strlen (gtk_entry_get_text (GTK_ENTRY (entry)))) {
 			prefix = g_strconcat (gtk_entry_get_text (GTK_ENTRY (entry)),
 					      event->string, NULL);
+		} else {
+			return FALSE;
+		}
 		
 		pos = strlen (prefix);
 		nprefix = NULL;
@@ -1199,10 +1189,13 @@ entry_event (GtkEditable    *entry,
 
 			g_free (nprefix);
 			g_free (prefix);
+			return TRUE;
 		}
+
+		g_free (prefix);
 	}
 	
-	return dialog->completion_started;
+	return FALSE;
 }
 
 static void
@@ -1444,7 +1437,7 @@ panel_run_dialog_new (GdkScreen *screen,
 	panel_run_dialog_setup_entry         (dialog, gui);
 	panel_run_dialog_setup_file_button   (dialog, gui);
 	panel_run_dialog_setup_program_list  (dialog, gui);
-	panel_run_dialog_setup_list_checkbox (dialog, gui);
+	panel_run_dialog_setup_list_expander (dialog, gui);
 
 	panel_run_dialog_update_content (dialog, panel_profile_get_show_program_list ());
 
@@ -1476,7 +1469,7 @@ panel_run_dialog_present (GdkScreen *screen)
 	GladeXML *gui;
 	char     *glade_file;
 
-	if (no_run_box)
+	if (panel_profile_get_inhibit_command_line ())
 		return;
 
 	if (static_dialog) {
