@@ -148,7 +148,6 @@ panel_size_change (GtkWidget *widget, gpointer data)
 			      widget);
 }
 
-#ifdef FIXME_FOR_NEW_CONFIG
 void
 back_change (AppletInfo  *info,
 	     PanelWidget *panel)
@@ -181,7 +180,6 @@ panel_back_change (GtkWidget *widget, gpointer data)
 	update_config_back(PANEL_WIDGET(widget));
 #endif /* FIXME_FOR_NEW_CONFIG */
 }
-#endif
 
 static void
 panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
@@ -194,9 +192,7 @@ panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 
 	orientation_change(info,PANEL_WIDGET(widget));
 	size_change(info,PANEL_WIDGET(widget));
-#ifdef FIXME_FOR_NEW_CONFIG
 	back_change(info,PANEL_WIDGET(widget));
-#endif
 }
 
 static void
@@ -236,37 +232,6 @@ menu_deactivate(GtkWidget *w, PanelData *pd)
 	panel_toplevel_unblock_auto_hide (PANEL_TOPLEVEL (pd->panel));
 }
 
-static void
-panel_remove_applets (PanelWidget *panel)
-{
-	GList *l;
-
-	for (l = panel->applet_list; l; l = l->next) {
-		AppletData *applet_data = l->data;
-		AppletInfo *info;
-
-		info = g_object_get_data (
-				G_OBJECT (applet_data->applet), "applet_info");
-
-		if (info && info->type == PANEL_OBJECT_LAUNCHER)
-			launcher_properties_destroy (info->data);
-			
-	}
-}
-
-static void
-panel_remove_open_create_launcher_dialogs (PanelWidget *panel)
-{
-	GSList *l;
-
-	for (l = panel->create_launcher_dialog_list; l ; l = l->next) {
-		GtkWidget *dialog = l->data;
-
-		if (GTK_IS_WIDGET (dialog))
-			gtk_widget_destroy (dialog);
-	}
-}
-
 static void
 panel_destroy (PanelToplevel *toplevel,
 	       PanelData     *pd)
@@ -275,10 +240,6 @@ panel_destroy (PanelToplevel *toplevel,
 
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 
-	panel_remove_applets (panel_widget);
-
-	panel_remove_open_create_launcher_dialogs (panel_widget);
-
 	if (pd->menu)
 		g_object_unref (pd->menu);
 	pd->menu = NULL;
@@ -372,7 +333,7 @@ panel_popup_menu (PanelToplevel *toplevel,
 			     gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-			panel_menu_position, panel_widget, button, activate_time);
+			panel_menu_position, toplevel, button, activate_time);
 
 	return TRUE;
 }
@@ -487,7 +448,8 @@ drop_menu (PanelWidget *panel,
 	if ( ! panel_profile_list_is_writable (PANEL_GCONF_OBJECTS))
 		return FALSE;
 
-	panel_menu_button_create (panel->toplevel, position, menu_path, menu_path != NULL);
+	panel_menu_button_create (panel->toplevel, position, menu_path,
+				  menu_path != NULL, menu_path);
 
 	return TRUE;
 }
@@ -543,7 +505,7 @@ drop_directory (PanelWidget *panel, int pos, const char *dir)
 
 	if (panel_is_program_in_path ("nautilus")) {
 		/* nautilus */
-		return drop_nautilus_uri (panel, pos, dir, "gnome-folder.png");
+		return drop_nautilus_uri (panel, pos, dir, "gnome-fs-directory.png");
 	} else {
 		return drop_menu (panel, pos, dir);
 	}
@@ -896,6 +858,10 @@ panel_check_drop_forbidden (PanelWidget    *panel,
 	if (!panel)
 		return FALSE;
 
+	if (panel_toplevel_get_locked_down (panel->toplevel) ||
+	    panel_profile_get_locked_down ())
+		return FALSE;
+
 	if (info == TARGET_APPLET_INTERNAL) {
 		GtkWidget *source_widget;
 
@@ -994,6 +960,13 @@ panel_receive_dnd_data (PanelWidget      *panel,
 			guint             time_)
 {
 	gboolean success = FALSE;
+
+	if (panel_toplevel_get_locked_down (panel->toplevel) ||
+	    panel_profile_get_locked_down ()) {
+		gtk_drag_finish (context, FALSE, FALSE, time_);
+		return;
+	}
+
 	switch (info) {
 	case TARGET_URL:
 		success = drop_urilist (panel, pos, (char *)selection_data->data);
@@ -1101,12 +1074,10 @@ panel_widget_setup(PanelWidget *panel)
 			  "applet_move",
 			  G_CALLBACK(panel_applet_move),
 			  NULL);
-#ifdef FIXME_FOR_NEW_TOPLEVEL
 	g_signal_connect (G_OBJECT (panel),
 			  "back_change",
 			  G_CALLBACK (panel_back_change),
 			  NULL);
-#endif
 	g_signal_connect (G_OBJECT (panel),
 			  "size_change",
 			  G_CALLBACK (panel_size_change),
@@ -1222,9 +1193,9 @@ panel_delete_without_query (PanelToplevel *toplevel)
 } 
 
 static void
-remove_panel_accept (GtkWidget     *dialog,
-		     int            response,
-		     PanelToplevel *toplevel)
+panel_deletion_response (GtkWidget     *dialog,
+			 int            response,
+			 PanelToplevel *toplevel)
 {
 	if (response == GTK_RESPONSE_OK) {
 		panel_push_window_busy (dialog);
@@ -1232,25 +1203,25 @@ remove_panel_accept (GtkWidget     *dialog,
 		panel_pop_window_busy (dialog);
 	}
 
-	g_object_set_data (G_OBJECT (toplevel), "panel-delete-dialog", NULL);
-
 	gtk_widget_destroy (dialog);
 }
 
 static void
-panel_delete_query (PanelToplevel *toplevel)
+panel_deletion_destroy_dialog (GtkWidget *widget,
+			       PanelToplevel *toplevel)
 {
+	panel_toplevel_unblock_auto_hide (toplevel);
+	g_object_set_data (G_OBJECT (toplevel), "panel-delete-dialog", NULL);
+}
+
+GtkWidget *
+panel_deletion_dialog (PanelToplevel *toplevel)
+{
+
 	GtkWidget *dialog;
 
-	dialog = g_object_get_data (G_OBJECT (toplevel), "panel-delete-dialog");
-
-	if (dialog) {
-		gtk_window_present (GTK_WINDOW (dialog));
-		return;
-	}
-
 	dialog = gtk_message_dialog_new (
-			GTK_WINDOW (toplevel), 
+			GTK_WINDOW (toplevel),
 			GTK_DIALOG_MODAL,
 			GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_NONE,
@@ -1267,20 +1238,44 @@ panel_delete_query (PanelToplevel *toplevel)
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"panel_remove_query", "Panel");
+
 	gtk_window_set_screen (GTK_WINDOW (dialog),
-			       gtk_window_get_screen (GTK_WINDOW (toplevel)));
+				gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
+	 g_signal_connect (dialog, "destroy",
+                           G_CALLBACK (panel_deletion_destroy_dialog),
+                           toplevel);
+
 	g_object_set_data (G_OBJECT (toplevel), "panel-delete-dialog", dialog);
 
+	return dialog;
+}
+
+static void
+panel_query_deletion (PanelToplevel *toplevel)
+{
+	GtkWidget *dialog;
+
+	dialog = g_object_get_data (G_OBJECT (toplevel), "panel-delete-dialog");
+
+	if (dialog) {
+		gtk_window_present (GTK_WINDOW (dialog));
+		return;
+	}
+
+	dialog = panel_deletion_dialog (toplevel);
+
 	g_signal_connect (dialog, "response",
-			  G_CALLBACK (remove_panel_accept),
+			  G_CALLBACK (panel_deletion_response),
 			  toplevel);
+
 	panel_signal_connect_object_while_alive (
 			G_OBJECT (toplevel), "destroy",
 			G_CALLBACK (gtk_widget_destroy),
 			G_OBJECT (dialog));
+
 	gtk_widget_show_all (dialog);
 }
 
@@ -1297,5 +1292,7 @@ panel_delete (PanelToplevel *toplevel)
 		return;
 	}
 
-	panel_delete_query (toplevel);
+	panel_toplevel_block_auto_hide (toplevel);
+
+	panel_query_deletion (toplevel);
 }
